"""
  ALL available activities that could be taken by a user, this list will be used while  naming pickle
  files
"""

available_activities = ['tweet_delete_events',
                    'direct_message_mark_read_events',
                    'direct_message_indicate_typing_events',
                    'retweet_to_user_events',
                    'retweet_by_user_events',
                    'tweet_create_events',
                    'reply_to_user_events',
                    'reply_by_user_events',
                    'mention_to_user_events',
                    'message_to_user_events',
                    'message_by_user_events',
                    'follow_to_user_events',
                    'follow_by_user_events',
                    'unfollow_by_user_events',
                    'block_by_user_events',
                    'unblock_by_user_events',
                    'mute_by_user_events',
                    'unmute_by_user_events',
                    'favorite_to_user_events',
                    'favorite_by_user_events']


from aws_s3_resource.s3_bucket import S3Bucket
from aws_s3_resource.s3_object import S3Object
from .activity_identifier import ActivityIdentifier
from .resources import available_activities
import pickle as pk
import os
import io
import time

class SaveToS3:
    """
            This class have tow main functions :
            1- initialize the storing medium (s3 bucket) in case that it does not exist
            2- save the activity:
             - call the activity identifier to determine the activity name
             - save it on its correct file
    """

    def  __init__(self, bucket_name):
        self.bucket_name = bucket_name

    @staticmethod
    def initialize_s3_storing_medium(bucket_name = None):
        if not S3Bucket.is_available(bucket_name):

            # Create a Bucket
            S3Bucket.create(bucket_name)
            for activity in available_activities:
                loggs = list()

                # Save empty list to each action log
                file_dict = {
                            'file_name': f'{activity}.pkl',
                            'file_bytes': pk.dumps(loggs) }

                S3Object.upload(bucket_name, file_dict, with_random_prefix=False)

    def save_activity(self,response):

        identified_activity = ActivityIdentifier().identify_activities(response= response)
        object_key  = f'{identified_activity}.pkl'
        logg_bytes = S3Object.download(self.bucket_name, object_key)

        with io.BytesIO(logg_bytes) as f:
            loggs = pk.load(f)

        saved_event = {f'{identified_activity}' : response}

        # Add current action to log
        saved_event['timestamp'] = int(time.time())
        loggs.append(saved_event)

        data_bytes = pk.dumps(loggs)

        file_dict = {
            'file_name': object_key,
            'file_bytes': data_bytes
        }

        S3Object.upload(self.bucket_name, file_dict, with_random_prefix=False)
        return f'successful {identified_activity}'




from .activity_identifier import ActivityIdentifier
from .resources import available_activities
import pickle as pk
import os


class SaveToFolder:

    """
        This class have tow main functions :
        1- initialize the storing medium (local folder) in case that it does not exist
        2- save the activity:
         - call the activity identifier to determine the activity name
         - save it on its correct file
    """

    def  __init__(self, events_folder ):
        self.events_folder = events_folder

    @staticmethod
    def initialize_local_storing_medium(events_folder = None):
        if not os.path.exists(events_folder):
            os.mkdir(events_folder)
            for activity in available_activities:
                loggs = list()
                with open(os.path.join(events_folder, f'{activity}.pkl'), 'wb') as f:
                    pk.dump(loggs, f)

    def save_activity(self,response):
        identified_activity = ActivityIdentifier().identify_activities(response=response)

        with open(os.path.join(self.events_folder, f'{identified_activity}.pkl'), 'rb') as f:
            events = pk.load(f)

        events.append(response)

        with open(os.path.join(self.events_folder, f'{identified_activity}.pkl'), 'wb') as f:
            pk.dump(events, f)

        return f'successful {identified_activity}'



from aws_s3_resource.s3_bucket import S3Bucket
from aws_s3_resource.s3_object import S3Object
import pickle as pk
import time
import os
import io

class ActivityIdentifier:
    """ The main purpose of this class is to extract the activity name"""
    basic_activities = ['tweet_delete_events', 'direct_message_mark_read_events',
                    'direct_message_indicate_typing_events']

    type_based_activities = ['follow_events','mute_events','block_events']

    # Conditons to determine who did the event
    by_to_conditions = {
        'by_if_equal': lambda id1, id2: 'by' if id1 == id2 else 'to',
        'to_if_equal': lambda id1, id2: 'to' if id1 == id2 else 'by'
    }

    @classmethod
    def identify_activities(cls, response):
        # Get activities list
        keys_list = response.keys()

        # Get activity name
        activity_name = ActivityIdentifier.get_activity_name(keys_list)

        if activity_name in cls.basic_activities:
            activity = activity_name

        else:
            # If activity depends on type
            if activity_name in cls.type_based_activities:

                # Get activity type
                activity_type = response[f'{activity_name}'][0]['type']

                # Test if the event done by user or to user
                if activity_type == 'follow':
                    follow_to = response['follow_events'][0]['target']['id']
                    user_id = response['for_user_id']
                    res = cls.by_to_conditions['to_if_equal'](user_id, follow_to)
                    activity = f'follow_{res}_user_events'

                # If event is unfollow, mute, unmute, block, or unblock
                else:
                    activity = f'{activity_type}_by_user_events'

            elif activity_name == 'direct_message_events':
                message_by = response["direct_message_events"][0]['message_create']['sender_id']
                user_id = response['for_user_id']
                res = cls.by_to_conditions['by_if_equal'](message_by, user_id)
                activity = f'message_{res}_user_events'

            elif activity_name == 'favorite_events':
                favorite_by = response['favorite_events'][0]['user']['id_str']
                user_id = response['for_user_id']
                res = cls.by_to_conditions['by_if_equal'](favorite_by, user_id)
                activity = f'favorite_{res}_user_events'

            elif activity_name == 'tweet_create_events':
                # Tweet create events maybe tweet (by user), re-tweet(to user,by user) , reply(to user,by user),mention (to user)
                if response['tweet_create_events'][0]['in_reply_to_status_id'] is None:
                   # Recognize the activity is it re-tweet or tweet by user ?
                   # If false it is tweet event
                    if "retweeted_status" in response['tweet_create_events'][0].keys():
                        retweeted_by = response['tweet_create_events'][0]['user']['id_str']
                        user_id = response['for_user_id']

                        res = cls.by_to_conditions['by_if_equal'](retweeted_by, user_id)
                        activity = f'retweet_{res}_user_events'

                    else:
                        activity = activity_name

                else:

                    if response['tweet_create_events'][0]['in_reply_to_user_id_str'] == response['for_user_id']:
                        activity = 'reply_to_user_events'

                    else:
                        if response['tweet_create_events'][0]['user']['id_str'] == response['for_user_id']:
                            activity = 'reply_by_user_events'
                        else:
                            activity = 'mention_to_user_events'

        return activity

    @staticmethod
    def get_activity_name(keys_list):
        for k in keys_list:
            if 'event' in k:
                return k


import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="twitter_activity_api_dummy",
    version="0.0.3",
    author="sammer sallam",
    author_email="samersallam92@gmail.com",
    description="This library is to help you work with web-hook, Classify twitter account activities and save these activities ",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Samer92/twitter_activity_api_dummy",
    packages=["twitter_activity_api_dummy/activity_manager","twitter_activity_api_dummy/webhook_manager"],
    install_requires=["aws-s3-resource==0.0.2","boto3==1.12.43","botocore==1.15.43","docutils==0.15.2",
                      "jmespath==0.9.5","python-dateutil==2.8.1","s3transfer==0.3.3","six==1.14.0",
                      "urllib3==1.25.9","certifi==2020.4.5.1","chardet==3.0.4","idna==2.9","oauthlib==3.1.0","PySocks==1.7.1",
                      "requests==2.23.0","requests-oauthlib==1.3.0","tweepy==3.8.0"],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)


import json
from tweepy.error import TweepError
from tweepy import OAuthHandler
import requests

class WebhooksManager:
    _protocol = "https:/"
    _host = "api.twitter.com"
    _version = "1.1"
    _product = "account_activity"

    def __init__(self, token_file=None, tokens_dict=None):
        if (token_file is not None) and (tokens_dict is not None):
            raise Exception('You can pass the tokens as json file or dictionary but not both together')

        elif token_file:
            with open(token_file) as f:
                twitter_tokens = json.load(f)

        elif tokens_dict:
            twitter_tokens = tokens_dict

        else:
            raise Exception('You have to pass the tokens as json file or dictionary')

        self._consumer_key = twitter_tokens['consumer_key']
        self._consumer_secret = twitter_tokens['consumer_secret']
        self._access_token = twitter_tokens['access_token']
        self._access_token_secret = twitter_tokens['access_token_secret']
        self._env_name = twitter_tokens['env_name']

        self._auth = OAuthHandler(
            bytes(self._consumer_key, 'ascii'), bytes(self._consumer_secret, 'ascii')
        )

        self._auth.set_access_token(
            bytes(self._access_token, 'ascii'), bytes(self._access_token_secret, 'ascii')
        )

    def api(self, method: str, endpoint: str, data: dict = None):
        """
        :param method: GET or POST
        :param endpoint: API Endpoint to be specified by user
        :param data: POST Request payload parameter
        :return: json
        """
        try:
            with requests.Session() as r:
                response = r.request(
                    url="/".join(
                        [
                            self._protocol,
                            self._host,
                            self._version,
                            self._product,
                            endpoint,
                        ]
                    ),
                    method=method,
                    auth=self._auth.apply_auth(),
                    data=data,
                )
                return response
        except TweepError:
            raise

    def register_webhook(self, callback_url: str):
        try:
            return self.api(
                method="POST",
                endpoint=f"all/{self._env_name}/webhooks.json",
                data={"url": callback_url},
            )
        except Exception:
            raise

    # Get all webhooks we have
    def get_webhooks(self):
        return self.api(
            method="GET",
            endpoint=f"all/webhooks.json",
        ).json()

    # Delete webhook by id
    def delete_webhook(self, id_):
        del_ = self.api(
            method="DELETE",
            endpoint=f"all/{self._env_name}/webhooks/{id_}.json",
        )

        return str(del_)

    def subscribe(self):
        try:
            return self.api(
                method="POST",
                endpoint=f"all/{self._env_name}/subscriptions.json",
            )
        except Exception:
            raise


from .save_to_s3 import SaveToS3
from .save_to_folder import SaveToFolder
import os


class SaveManager:
    """ This class works on :
         1- determine the storing medium and call the setup function depends on that at the first of running the app.
        2- call the correct save function to process and save the activity

    """


    def __init__(self, storing_medium='local', events_folder=os.getcwd(), bucket_name=None):

        self.storing_medium = storing_medium

        if self.storing_medium == 'local':
            self.events_folder = events_folder
            SaveToFolder.initialize_local_storing_medium(events_folder=events_folder)
            self.save_medium = SaveToFolder(self.events_folder)

        elif self.storing_medium == 's3':
            self.bucket_name = bucket_name
            SaveToS3.initialize_s3_storing_medium(bucket_name=bucket_name)
            self.save_medium = SaveToS3(self.bucket_name)






